#include "mbed.h"
#include "SPI.h"
#include "r_bsp_cmn.h"
#include "dma_if.h"
#include "rspi_iodefine.h"
// ROHM BH1745 driver include
#include "rohm-sensor-hal/rohm-sensor-hal/rohm_hal.h"       //mbed.h, types, DEBUG_print*
#include "rohm-sensor-hal/rohm-sensor-hal/I2CCommon.h"
#include "rohm-bh1745/rohm-bh1745/bh1745_driver.h"
#include "rohm-bh1745/rohm-bh1745/bh1745.h"
#include "WS2812.h"
#include "PixelArray.h"

DigitalOut led1(LED1);

// Switch
DigitalIn sw(P2_15);

// Stepper motor control
DigitalOut enable(D8);
DigitalOut dir_z(D7);
DigitalOut dir_y(D6);
DigitalOut dir_x(D5);
DigitalOut step_z(D4);
DigitalOut step_y(D3);
DigitalOut step_x(D2);

#define START_WAIT      (2)
#define ENABLE          (0)
#define DISABLE         (1)
#define FOWARD          (1)
#define REVERSE         (0)

#define MICRO_STEP      (8)
#define STANDARD_STEP   (33)
#define TARGET_STEP     (MICRO_STEP * STANDARD_STEP)

#define MAX_COLOR       (5)

// NeoPixel
#define WS2812_BUF 64
#define NUM_COLORS 6
#define NUM_LEDS_PER_COLOR 10
#define MOSI P11_14
#define MISO P11_15
#define SCK  P11_12
 
PixelArray px(WS2812_BUF);
 
// See the program page for information on the timing numbers
// The given numbers are for the K64F
WS2812 ws(MOSI, MISO, SCK, WS2812_BUF, 0, 0, 0, 0);
 
int bh1745_print_one_value(uint16_t *color){
    bool error;
    uint16_t data[4] = {0,0,0,0};       //4 channels of 16-bit data

    error = bh1745_read_data(&data[0]);
    if (!error) {
        printf("1: Red[%4u], Green[%4u], Blue[%4u], Clear[%4u]\n\r", data[0], data[1], data[2], data[3]);
        return 0;
    }
    else {
        printf("\n\r");
        return 1;
    }
}

int main() {
    int target_step = 0;
    uint16_t *color = NULL;

    printf("Start EnoPick...\r\n\r\n");

    //
    // Color sensor setup
    //
    I2CCommonBegin();
    bh1745_wait_until_found();
    printf("\nSensor found.\n\r");
    bh1745_initial_setup();
    wait(1);

    //
    // NeoPixel Setuo
    //
    ws.useII(WS2812::PER_PIXEL); // use per-pixel intensity scaling

    // Stepper motor controller setup
    // disable
    enable = DISABLE;

    // default direction
    dir_x = FOWARD;
    dir_y = FOWARD;
    dir_z = FOWARD;

    // step low
    step_x = 0;
    step_y = 0;
    step_z = 0;

    // Wait sequence
    printf("Please wait %ds\r\n", START_WAIT);
    for(int time = 0; time < START_WAIT; time++){
        wait(1);
        printf("%d\r\n", time);
    }
    printf("\r\n\r\n");

    // Enable sequence
    enable = ENABLE;

    while (true) {
        if(bh1745_print_one_value(color) != 0) {
            continue;
        }

        // for each of the colours (j) write out 10 of them
        // the pixels are written at the colour*10, plus the colour position
        // all modulus 60 so it wraps around
        for (int i = 0; i < WS2812_BUF; i++) {
            px.SetR(i, color[0]);
            px.SetG(i, color[1]);
            px.SetB(i, color[2]);
            px.SetI(i, color[3]);
        }
        
        for (int z=WS2812_BUF; z >= 0 ; z--) {
            ws.write_offsets(px.getBuf(),z,z,z);
            wait(0.075);
        }

        if(sw == 0) {
            continue;
        }

        printf("your color => R[%d] G[%d] B[%d] I[%d]",
                color[0], color[1], color[2], color[3]);

        for(int color_cnt = 0; color_cnt < 3; color_cnt++){
            target_step = TARGET_STEP * color_cnt;
            if(target_step == 0){
                continue;
            }

            printf("\ttarget_step = %d\r\n", target_step);

            // default direction
            dir_x = FOWARD;
            dir_y = FOWARD;
            dir_z = FOWARD;

            for(int i = 0; i < target_step; i++){
                led1 = !led1;
                step_x = 1;
                step_y = 1;
                wait(0.001);
                step_x = 0;
                step_y = 0;
                wait(0.001);
            }

            // Wait 3s
            wait(3);

            // 
            dir_x = REVERSE;
            dir_y = REVERSE;
            dir_z = REVERSE;

            for(int i = 0; i < target_step; i++){
                led1 = !led1;
                step_x = 1;
                step_y = 1;
                wait(0.001);
                step_x = 0;
                step_y = 0;
                wait(0.001);
            }
        }
    }
}

